package com.aurellem.send;

import java.nio.ByteBuffer;

public class AudioSend {

	private final long deviceID;
	
	public AudioSend(long deviceID){
		this.deviceID = deviceID;
	}
	
	/** This establishes the LWJGL context as the context which will be copies to all 
	 *  other contexts.  It must be called before any calls to <code>addListener();</code>
	 */
	public void initDevice(){
		ninitDevice(this.deviceID);}
	public static native void ninitDevice(long device);
	
	/**
	 * The send device does not automatically process sound.  This step function will cause 
	 * the desired number of samples to be processed for each listener.  The results will then 
	 * be available via calls to <code>getSamples()</code> for each listener.
	 * @param samples
	 */
	public void step(int samples){
		nstep(this.deviceID, samples);}
	public static native void nstep(long device, int samples);

	/**
	 * Retrieve the final rendered sound for a particular listener.  <code>contextNum == 0</code>
	 * is the main LWJGL context.
	 * @param buffer
	 * @param samples
	 * @param contextNum
	 */
	public void getSamples(ByteBuffer buffer, int samples, int contextNum){
		ngetSamples(this.deviceID, buffer, buffer.position(), samples, contextNum);}
	public static native void ngetSamples(
			long device, ByteBuffer buffer, int position, int samples, int contextNum);
	
	/**
	 * Create an additional listener on the recorder device.  The device itself will manage 
	 * this listener and synchronize it with the main LWJGL context. Processed sound samples
	 * for this listener will be available via a call to <code>getSamples()</code> with 
	 * <code>contextNum</code> equal to the number of times this method has been called. 
	 */
	public void addListener(){naddListener(this.deviceID);}
	public static native void naddListener(long device);
	
	/**
	 * This will internally call <code>alListener3f<code> in the appropriate slave context and update
	 * that context's listener's parameters. Calling this for a number greater than the current 
	 * number of slave contexts will have no effect.
	 * @param pname
	 * @param v1
	 * @param v2
	 * @param v3
	 * @param contextNum
	 */
	public void setNthListener3f(int pname, float v1, float v2, float v3, int contextNum){
		nsetNthListener3f(pname, v1, v2, v3, this.deviceID, contextNum);}
	public static native void 
	nsetNthListener3f(int pname, float v1, float v2, float v3, long device, int contextNum);
	
	/**
	 * This will internally call <code>alListenerf<code> in the appropriate slave context and update
	 * that context's listener's parameters. Calling this for a number greater than the current 
	 * number of slave contexts will have no effect.
	 * @param pname
	 * @param v1
	 * @param contextNum
	 */
	public void setNthListenerf(int pname, float v1, int contextNum){
		nsetNthListenerf(pname, v1, this.deviceID, contextNum);}
	public static native void nsetNthListenerf(int pname, float v1, long device, int contextNum);
	
}
